@@ -3,7 +3,6 @@ Creation and extension of validators, with implementations for existing drafts.
 """
 from __future__ import annotations
 
-from collections import deque
 from collections.abc import Sequence
 from functools import lru_cache
 from urllib.parse import unquote, urldefrag, urljoin, urlsplit
@@ -18,6 +17,7 @@ import warnings
 import attr
 
 from jsonschema import (
+    _format,
     _legacy_validators,
     _types,
     _utils,
@@ -108,7 +108,8 @@ def create(
     meta_schema,
     validators=(),
     version=None,
-    type_checker=_types.draft7_type_checker,
+    type_checker=_types.draft202012_type_checker,
+    format_checker=_format.draft202012_format_checker,
     id_of=_id_of,
     applicable_validators=lambda schema: schema.items(),
 ):
@@ -149,6 +150,14 @@ def create(
             If unprovided, a `jsonschema.TypeChecker` will be created
             with a set of default types typical of JSON Schema drafts.
 
+        format_checker (jsonschema.FormatChecker):
+
+            a format checker, used when applying the :validator:`format`
+            validator.
+
+            If unprovided, a `jsonschema.FormatChecker` will be created
+            with a set of default formats typical of JSON Schema drafts.
+
         id_of (collections.abc.Callable):
 
             A function that given a schema, returns its ID.
@@ -163,6 +172,8 @@ def create(
 
         a new `jsonschema.protocols.Validator` class
     """
+    # preemptively don't shadow the `Validator.format_checker` local
+    format_checker_arg = format_checker
 
     @attr.s
     class Validator:
@@ -170,6 +181,7 @@ def create(
         VALIDATORS = dict(validators)
         META_SCHEMA = dict(meta_schema)
         TYPE_CHECKER = type_checker
+        FORMAT_CHECKER = format_checker_arg
         ID_OF = staticmethod(id_of)
 
         schema = attr.ib(repr=reprlib.repr)
@@ -284,7 +296,13 @@ def create(
     return Validator
 
 
-def extend(validator, validators=(), version=None, type_checker=None):
+def extend(
+    validator,
+    validators=(),
+    version=None,
+    type_checker=None,
+    format_checker=None,
+):
     """
     Create a new validator class by extending an existing one.
 
@@ -322,6 +340,14 @@ def extend(validator, validators=(), version=None, type_checker=None):
             If unprovided, the type checker of the extended
             `jsonschema.protocols.Validator` will be carried along.
 
+        format_checker (jsonschema.FormatChecker):
+
+            a format checker, used when applying the :validator:`format`
+            validator.
+
+            If unprovided, the format checker of the extended
+            `jsonschema.protocols.Validator` will be carried along.
+
     Returns:
 
         a new `jsonschema.protocols.Validator` class extending the one
@@ -343,11 +369,14 @@ def extend(validator, validators=(), version=None, type_checker=None):
 
     if type_checker is None:
         type_checker = validator.TYPE_CHECKER
+    if format_checker is None:
+        format_checker = validator.FORMAT_CHECKER
     return create(
         meta_schema=validator.META_SCHEMA,
         validators=all_validators,
         version=version,
         type_checker=type_checker,
+        format_checker=format_checker,
         id_of=validator.ID_OF,
     )
 
@@ -378,6 +407,7 @@ Draft3Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft3_type_checker,
+    format_checker=_format.draft3_format_checker,
     version="draft3",
     id_of=lambda schema: schema.get("id", ""),
     applicable_validators=_legacy_validators.ignore_ref_siblings,
@@ -414,6 +444,7 @@ Draft4Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft4_type_checker,
+    format_checker=_format.draft4_format_checker,
     version="draft4",
     id_of=lambda schema: schema.get("id", ""),
     applicable_validators=_legacy_validators.ignore_ref_siblings,
@@ -455,6 +486,7 @@ Draft6Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft6_type_checker,
+    format_checker=_format.draft6_format_checker,
     version="draft6",
     applicable_validators=_legacy_validators.ignore_ref_siblings,
 )
@@ -496,6 +528,7 @@ Draft7Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft7_type_checker,
+    format_checker=_format.draft7_format_checker,
     version="draft7",
     applicable_validators=_legacy_validators.ignore_ref_siblings,
 )
@@ -541,6 +574,7 @@ Draft201909Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft201909_type_checker,
+    format_checker=_format.draft201909_format_checker,
     version="draft2019-09",
 )
 
@@ -586,6 +620,7 @@ Draft202012Validator = create(
         "uniqueItems": _validators.uniqueItems,
     },
     type_checker=_types.draft202012_type_checker,
+    format_checker=_format.draft202012_format_checker,
     version="draft2020-12",
 )
 
@@ -767,16 +802,16 @@ class RefResolver(object):
     def _find_in_referrer(self, key):
         return self._get_subschemas_cache()[key]
 
-    @lru_cache()
+    @lru_cache()  # noqa: B019
     def _get_subschemas_cache(self):
         cache = {key: [] for key in _SUBSCHEMAS_KEYWORDS}
-        for keyword, subschema in _search_schema(
+        for keyword, subschema in _utils.search_schema(
             self.referrer, _match_subschema_keywords,
         ):
             cache[keyword].append(subschema)
         return cache
 
-    @lru_cache()
+    @lru_cache()  # noqa: B019
     def _find_in_subschemas(self, url):
         subschemas = self._get_subschemas_cache()["$id"]
         if not subschemas:
@@ -844,7 +879,10 @@ class RefResolver(object):
         else:
 
             def find(key):
-                yield from _search_schema(document, _match_keyword(key))
+                yield from _utils.search_schema(
+                    document,
+                    _utils.match_keyword(key),
+                )
 
         for keyword in ["$anchor", "$dynamicAnchor"]:
             for subschema in find(keyword):
@@ -930,32 +968,12 @@ class RefResolver(object):
 _SUBSCHEMAS_KEYWORDS = ("$id", "id", "$anchor", "$dynamicAnchor")
 
 
-def _match_keyword(keyword):
-
-    def matcher(value):
-        if keyword in value:
-            yield value
-
-    return matcher
-
-
 def _match_subschema_keywords(value):
     for keyword in _SUBSCHEMAS_KEYWORDS:
         if keyword in value:
             yield keyword, value
 
 
-def _search_schema(schema, matcher):
-    """Breadth-first search routine."""
-    values = deque([schema])
-    while values:
-        value = values.pop()
-        if not isinstance(value, dict):
-            continue
-        yield from matcher(value)
-        values.extendleft(value.values())
-
-
 def validate(instance, schema, cls=None, *args, **kwargs):
     """
     Validate an instance under the given schema.
